--[[
--
-- Ideas
--  (which means they are *ideas* - NOT necessarily things I *want* to be
--  implemented)
--
--  - Some sort of text progress bar like [#####       ] (or just force ckk to
--  implement :AddFrame)
--  - Fix coloring.
--  - Try to flash or increase font size of individual lines as they near
--  completion.
--  - Ability to collapse/expand a module like QuestsFu does with zones in its
--  tablet.
--  - Some sort of magic when you click a timer. For example use it to
--  increase/decrease font size of that line or freeze it in place so that it's
--  not removed when the timer reaches 0 or whatever.
--  - Set background color according to severity
--  - Set up a hash-based coloring system for the module names, so that we're
--  almost guaranteed to have all the modules listed in different colors but
--  they should have the same color every time. I believe Firefox has a few tab
--  addons that do this, and one is called ChromaTabs.
--]]


local name = "BigWigs Tablet Bars"
local L = AceLibrary("AceLocale-2.2"):new(name)
local tablet = AceLibrary("Tablet-2.0")
local bars = nil

local table_insert = table.insert
local table_sort = table.sort
local ipairs = ipairs
local pairs = pairs
local next = next

L:RegisterTranslations("enUS", function() return {
	["tabletbars"] = true,
	["Tablet Bars"] = true,
	["Alternative bar display for Big Wigs."] = true,
} end )

--Chinese Translate by 月色狼影@CWDG
--CWDG site: http://Cwowaddon.com
L:RegisterTranslations("zhCN", function() return {
	["tabletbars"] = "计时条列表",
	["Tablet Bars"] = "计时条列表",
	["Alternative bar display for Big Wigs."] = "切换BigWigs计时条显示方式",
} end )

local addon = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "FuBarPlugin-2.0")
_G.BWTabletBars = addon
addon.hasIcon = "Interface\\Icons\\INV_Gizmo_01"

-------------------------------------------------------------------------------
-- Event and initialization                                                  --
-------------------------------------------------------------------------------

function addon:OnInitialize()
	addon:RegisterDB("BigWigsTimerBarsDB")
	addon:RegisterDefaults("profile", {
		wtf = "lol",
	})

	addon.hideWithoutStandby = true
	addon.hasNoColor = true
	addon.hasNoText = true
	addon.defaultMinimapPosition = 60
	addon.clickableTooltip = true
	addon.defaultPosition = "RIGHT"
	--addon.tooltipHiddenWhenEmpty = true
end

function addon:OnEnable()
	self:RegisterEvent("BigWigs_StartBar")
	self:RegisterEvent("BigWigs_StopBar")
	self:RegisterEvent("AceEvent_FullyInitialized")
end

function addon:AceEvent_FullyInitialized()
	DEFAULT_CHAT_FRAME:AddMessage("|cffff0000BigWigs Tablet Bars is a proof-of-concept addon and it is not meant for use. If anyone wants to make it better and usable, be my guest.|r")
end

local function rescheduleUpdates()
	if next(bars) then
		addon:ScheduleRepeatingEvent("displayUpdate", addon.UpdateDisplay, 0.4, addon)
	else
		addon:ScheduleRepeatingEvent("displayUpdate", addon.UpdateDisplay, 60, addon)
	end
end

local function repeating()
	for mod, barlist in pairs(bars) do
		for i, tbl in ipairs(barlist) do
			tbl.time = tbl.time - 1
		end
	end
end

local function timeSorter(a, b)
	if a.time < b.time then return true
	else return false end
end

function addon:BigWigs_StartBar(mod, text, time, icon)
	if not text or not time then return end
	local key = mod.name or mod:ToString()
	if not key then return end
	if not bars then bars = {} end
	if not bars[key] then bars[key] = {} end
	table_insert(bars[key], { text = text, time = time, icon = icon })
	table_sort(bars[key], timeSorter)
	self:ScheduleRepeatingEvent("BWTBCountdown", repeating, 1, self)
	rescheduleUpdates()
end

function addon:BigWigs_StopBar(mod, text)
	local key = mod.name or mod:ToString()
	if bars[key] then
		for i, v in ipairs(bars[key]) do
			if v.text == text then
				table.remove(bars[key], i)
				break
			end
		end
	end
	rescheduleUpdates()
end

-------------------------------------------------------------------------------
-- Tablet handling                                                           --
-------------------------------------------------------------------------------

do
	local moduleCategories = {}
	function addon:OnTooltipUpdate()
		if not bars then return end
		for k, v in pairs(bars) do
			table_insert(moduleCategories, k)
		end
		table_sort(moduleCategories)
		for i, category in ipairs(moduleCategories) do
			local cat = tablet:AddCategory(
				"columns", 2,
				"text", category,
				"child_justify", "LEFT",
				"child_justify2", "RIGHT",
				"child_wrap", false,
				"child_wrap2", false,
				"child_hasCheck", true,
				"child_checked", true,
				"showWithoutChildren", false
			)
			for i, v in ipairs(bars[category]) do
				if v.time >= 0 then
					cat:AddLine(
						"text", v.text,
						"text2", v.time,
						"checkIcon", v.icon
					)
				else
					table.remove(bars[category], i)
				end
			end
			if #bars[category] == 0 then
				bars[category] = nil
			end
		end
		rescheduleUpdates()
		for k in pairs(moduleCategories) do moduleCategories[k] = nil end
	end
end

